/*
 * $Id: BoardSwapper.java 8554 2013-02-19 06:51:41Z swampwallaby $
 *
 * Copyright (c) 2000-2003 by Rodney Kinney
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License (LGPL) as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, copies are available
 * at http://www.opensource.org.
 */
package VASL.build.module.map;

import java.awt.Component;
import java.awt.Point;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;

import VASL.build.module.map.boardPicker.ASLBoard;
import VASSAL.build.AbstractBuildable;
import VASSAL.build.Buildable;
import VASSAL.build.GameModule;
import VASSAL.build.module.Map;
import VASSAL.build.module.map.BoardPicker;
import VASSAL.build.module.map.GlobalMap;
import VASSAL.counters.GamePiece;
import VASSAL.tools.imageop.Op;

/**
 * Allows the user the change boards in a Map window while
 * keeping the positions of all the counters on the map
 */
/* Referenced from VSQL buildfile */
public class BoardSwapper extends AbstractBuildable {
  private Map map;

  private final List<GamePiece> pieces = new ArrayList<>();
  private final List<Point> positions = new ArrayList<>();
  private final List<String> boards = new ArrayList<>();

  public void addTo(Buildable b) {
    map = (Map) b;
    JButton launch = new JButton("Change boards");
    launch.setAlignmentY(0.0F);
    launch.addActionListener(evt -> {
      recordPiecePositions();
      final ASLBoardPicker picker = new Picker(map);
      final JDialog d = new JDialog(GameModule.getGameModule().getPlayerWindow(),true);
      d.getContentPane().setLayout(new BoxLayout(d.getContentPane(),BoxLayout.Y_AXIS));
      d.getContentPane().add(picker.getControls());
      JButton okButton = new JButton("Ok");
      okButton.addActionListener(arg0 -> {
        d.setVisible(false);
        picker.finish();
      });
      JButton cancelButton = new JButton("Cancel");
      cancelButton.addActionListener(arg0 -> d.setVisible(false));
      Box buttonBox = Box.createHorizontalBox();
      buttonBox.add(okButton);
      buttonBox.add(cancelButton);
      d.getContentPane().add(buttonBox);
      d.pack();
      d.setLocationRelativeTo(GameModule.getGameModule().getPlayerWindow());
      d.setVisible(true);
      restorePiecePositions();
      map.repaint();
    });
    try {
      launch.setIcon(new ImageIcon(Op.load("newBoards.gif").getImage(null)));
      launch.setText("");
      launch.setToolTipText("Pick new boards for this scenario");
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    map.getToolBar().add(launch);
  }

  protected void recordPiecePositions() {
    pieces.clear();
    positions.clear();
    boards.clear();

    GamePiece[] piece = map.getPieces();
    for (GamePiece gamePiece : piece) {
      ASLBoard b = (ASLBoard) map.findBoard(gamePiece.getPosition());
      if (b != null) {
        Point p = new Point(gamePiece.getPosition().x - b.bounds().x,
          gamePiece.getPosition().y - b.bounds().y);
        p = b.localCoordinates(p);
        pieces.add(gamePiece);
        boards.add(b.getName());
        positions.add(p);
      }
    }
  }

  protected void restorePiecePositions() {
    if (pieces.size() > 0) {
      for (int n = 0; n < pieces.size(); ++n) {
        ASLBoard b = null;
        if (boards.get(n) != null) {
          String boardName = boards.get(n);
          GamePiece piece = pieces.get(n);
          for (VASSAL.build.module.map.boardPicker.Board value : map.getBoards()) {
            ASLBoard board = (ASLBoard) value;
            if (boardName.equals(board.getName())) {
              b = board;
              break;
            }
          }
          if (b != null) {
            Point p = b.globalCoordinates(positions.get(n));
            piece.setPosition(new Point(p.x + b.bounds().x,
                                        p.y + b.bounds().y));
          }
        }
      }
    }
  }

  public String[] getAttributeNames() {
    return new String[0];
  }

  public String getAttributeValueString(String attName) {
    return null;
  }

  public void setAttribute(String attName, Object value) {
  }

  private static class Picker extends ASLBoardPicker {
    public Picker(Map m) {
      this.map = m;
      allowMultiple = true;
      setBoardDir((File) GameModule.getGameModule().getPrefs().getValue(BOARD_DIR));
    }

    public void finish() {
      super.finish();
      for (Object o : GameModule.getGameModule().getGameState().getGameComponents()) {
        if (o instanceof GlobalMap) {
          ((GlobalMap) o).setup(true);
        }
      }
      for (CASLThread t : this.map.getComponentsOf(CASLThread.class)) {
        t.setup(false);
        t.setup(true);
      }
      for (BoardPicker p : this.map.getComponentsOf(BoardPicker.class)) {
        ((ASLBoardPicker)p).setGlobalMapScale();
        new SetBoards(p, currentBoards).execute();
      }
    }

    @Override
    public Component getControls() {
      Component c = super.getControls();
      setBoards(this.map.getBoards());
      return c;
    }
  }
}
